@@ -1,4 +1,5 @@
 import hmac
+import pickle
 import importlib
 import inspect
 import logging
@@ -11,11 +12,13 @@ from bs4.element import Tag
 from bs4.formatter import HTMLFormatter
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import Model, QuerySet
+from django.db.models import Model
 from django.template.response import TemplateResponse
 from django.utils.safestring import mark_safe
 from django.views.generic.base import TemplateView
 
+from . import serializer
+
 
 logger = logging.getLogger(__name__)
 
@@ -35,7 +38,7 @@ class UnicornField:
         return self.__dict__
 
 
-class ComponentNotFoundError(Exception):
+class ComponentLoadError(Exception):
     pass
 
 
@@ -154,14 +157,17 @@ class UnicornTemplateResponse(TemplateResponse):
 
 class UnicornView(TemplateView):
     response_class = UnicornTemplateResponse
-    component_name = ""
+    component_name: str = ""
     request = None
 
     # Caches to reduce the amount of time introspecting the class
-    _methods_cache = None
-    _attribute_name_cache = None
+    _methods_cache: Dict[str, Callable] = {}
+    _attribute_names_cache: List[str] = []
     _hook_methods_cache: List[str] = []
 
+    # Dictionary with key: attribute name; value: pickled attribute value
+    _resettable_attributes_cache: Dict[str, Any] = {}
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
 
@@ -203,6 +209,26 @@ class UnicornView(TemplateView):
         self._attribute_names_cache = self._attribute_names()
         self._set_hook_methods_cache()
         self._methods_cache = self._methods()
+        self._set_resettable_attributes_cache()
+
+    def reset(self):
+        for (
+            attribute_name,
+            pickled_value,
+        ) in self._resettable_attributes_cache.items():
+            try:
+                attribute_value = pickle.loads(pickled_value)
+                self._set_property(attribute_name, attribute_value)
+            except TypeError:
+                logger.warn(
+                    f"Resetting '{attribute_name}' attribute failed because it could not be constructed."
+                )
+                pass
+            except pickle.PickleError:
+                logger.warn(
+                    f"Resetting '{attribute_name}' attribute failed because it could not be de-pickled."
+                )
+                pass
 
     def mount(self):
         """
@@ -244,7 +270,7 @@ class UnicornView(TemplateView):
         """
         Renders a UnicornView component with the public properties available. Delegates to a
         UnicornTemplateResponse to actually render a response.
-
+ 
         Args:
             param init_js: Whether or not to include the Javascript required to initialize the component.
         """
@@ -271,45 +297,6 @@ class UnicornView(TemplateView):
         Get publicly available properties and output them in a string-encoded JSON object.
         """
 
-        def _get_model_json(obj):
-            """
-            Serializes Django models.
-
-            Tried to use the Django serializer, but it failed for some field types.
-            """
-            model_field_names = [field.name for field in obj._meta.get_fields()]
-
-            model_json = {}
-            for field_name in model_field_names:
-                model_json[field_name] = getattr(obj, field_name)
-
-            return model_json
-
-        def _json_serializer(obj):
-            """
-            Handle the objects that the `orjson` deserializer can't handle automatically.
-
-            The types handled by `orjson` by default: dataclass, datetime, enum, float, int, numpy, str, uuid.
-            The types handled in this class: Django Model, Django QuerySet, any object with `to_json` method.
-
-            TODO: Investigate other ways to serialize objects automatically.
-            e.g. using DRF serializer: https://www.django-rest-framework.org/api-guide/serializers/#serializing-objects
-            """
-            if isinstance(obj, Model):
-                return _get_model_json(obj)
-            elif isinstance(obj, QuerySet):
-                queryset_json = []
-
-                for model in obj:
-                    model_json = _get_model_json(model)
-                    queryset_json.append(model_json)
-
-                return queryset_json
-            elif hasattr(obj, "to_json"):
-                return obj.to_json()
-
-            raise TypeError
-
         frontend_context_variables = {}
         attributes = self._attributes()
         frontend_context_variables.update(attributes)
@@ -329,9 +316,9 @@ class UnicornView(TemplateView):
                         value = field.widget.format_value(cleaned_value)
                         frontend_context_variables[key] = value
 
-        encoded_frontend_context_variables = orjson.dumps(
-            frontend_context_variables, default=_json_serializer,
-        ).decode("utf-8")
+        encoded_frontend_context_variables = serializer.dumps(
+            frontend_context_variables
+        )
 
         return encoded_frontend_context_variables
 
@@ -409,7 +396,7 @@ class UnicornView(TemplateView):
 
         return self.errors
 
-    def _attribute_names(self):
+    def _attribute_names(self) -> List[str]:
         """
         Gets publicly available attribute names. Cached in `_attribute_names_cache`.
         """
@@ -477,6 +464,9 @@ class UnicornView(TemplateView):
         return methods
 
     def _set_hook_methods_cache(self) -> None:
+        """
+        Caches the updating/updated attribute function names defined on the component.
+        """
         self._hook_methods_cache = []
 
         for attribute_name in self._attribute_names_cache:
@@ -488,6 +478,35 @@ class UnicornView(TemplateView):
                 if hasattr(self, function_name):
                     self._hook_methods_cache.append(function_name)
 
+    def _set_resettable_attributes_cache(self) -> None:
+        """
+        Caches the attributes that are "resettable" in `_resettable_attributes_cache`.
+        Cache is a dictionary with key: attribute name; value: pickled attribute value
+
+        Examples:
+            - `UnicornField`
+            - Django Models without a defined pk
+        """
+        self._resettable_attributes_cache = {}
+
+        for attribute_name, attribute_value in self._attributes().items():
+            if isinstance(attribute_value, UnicornField):
+                self._resettable_attributes_cache[attribute_name] = pickle.dumps(
+                    attribute_value
+                )
+            elif isinstance(attribute_value, Model):
+                if not attribute_value.pk:
+                    if attribute_name not in self._resettable_attributes_cache:
+                        try:
+                            self._resettable_attributes_cache[
+                                attribute_name
+                            ] = pickle.dumps(attribute_value)
+                        except pickle.PickleError:
+                            logger.warn(
+                                f"Caching '{attribute_name}' failed because it could not be pickled."
+                            )
+                            pass
+
     def _is_public(self, name: str) -> bool:
         """
         Determines if the name should be sent in the context.
@@ -517,6 +536,7 @@ class UnicornView(TemplateView):
             # Component methods
             "component_id",
             "component_name",
+            "reset",
             "mount",
             "hydrate",
             "updating",
@@ -556,7 +576,7 @@ class UnicornView(TemplateView):
         
         Returns:
             Instantiated `UnicornView` component.
-            Raises `ComponentNotFoundError` if the component cannot be found.
+            Raises `ComponentLoadError` if the component could not be loaded.
         """
         if component_id and use_cache:
             key = f"{component_name}-{component_id}"
@@ -571,31 +591,17 @@ class UnicornView(TemplateView):
                 component_name=component_name, component_id=component_id
             )
 
-            if not use_cache:
-                #  Re-initializes custom classes so that `reset` magic method will "clear" them as expected
-                _attributes = component._attributes()
+            #  Re-initializes classes so that `reset` magic method will "clear" them as expected
+            component.reset()
 
-                for attribute_name in _attributes:
-                    attribute = _attributes[attribute_name]
-
-                    if isinstance(attribute, UnicornField):
-                        try:
-                            component._set_property(
-                                attribute_name, attribute.__class__()
-                            )
-                        except TypeError:
-                            logger.warn(
-                                f"Resetting '{attribute_name}' attribute failed because it could not be constructed."
-                            )
-                            pass
-
-                component.hydrate()
+            component.hydrate()
 
             if component_id:
                 key = f"{component_name}-{component_id}"
                 constructed_views_cache[key] = component
 
             component._validate_called = False
+
             return component
 
         locations = []
@@ -650,6 +656,6 @@ class UnicornView(TemplateView):
             except AttributeError as e:
                 last_exception = e
 
-        raise ComponentNotFoundError(
-            f"'{component_name}' component could not be found."
+        raise ComponentLoadError(
+            f"'{component_name}' component could not be loaded."
         ) from last_exception
