@@ -1,4 +1,5 @@
 import { Attribute } from "./attribute.js";
+import { isEmpty, generateDbKey, hasValue } from "./utils.js";
 
 /**
  * Encapsulate DOM element for Unicorn-related information.
@@ -17,12 +18,19 @@ export class Element {
     this.isUnicorn = false;
     this.attributes = [];
     this.value = this.getValue();
+    this.parent = null;
+
+    if (this.el.parentElement) {
+      this.parent = new Element(this.el.parentElement);
+    }
 
     this.model = {};
     this.poll = {};
     this.actions = [];
+    this.db = {};
+    this.field = {};
 
-    this.key = undefined;
+    this.key = null;
     this.errors = [];
 
     if (!this.el.attributes) {
@@ -42,7 +50,21 @@ export class Element {
         this.model.eventType = attribute.modifiers.lazy ? "blur" : "input";
         this.model.isLazy = !!attribute.modifiers.lazy;
         this.model.isDefer = !!attribute.modifiers.defer;
-        this.model.debounceTime = attribute.modifiers.debounce ? parseInt(attribute.modifiers.debounce, 10) || -1 : -1;
+        this.model.debounceTime = attribute.modifiers.debounce
+          ? parseInt(attribute.modifiers.debounce, 10) || -1
+          : -1;
+      } else if (attribute.isField) {
+        this.field.name = attribute.value;
+        this.field.eventType = attribute.modifiers.lazy ? "blur" : "input";
+        this.field.isLazy = !!attribute.modifiers.lazy;
+        this.field.isDefer = !!attribute.modifiers.defer;
+        this.field.debounceTime = attribute.modifiers.debounce
+          ? parseInt(attribute.modifiers.debounce, 10) || -1
+          : -1;
+      } else if (attribute.isDb) {
+        this.db.name = attribute.value;
+      } else if (attribute.isPK) {
+        this.db.pk = attribute.value;
       } else if (attribute.isPoll) {
         this.poll.method = attribute.value ? attribute.value : "refresh";
         this.poll.timing = 2000;
@@ -84,6 +106,46 @@ export class Element {
         this.errors.push({ code, message: attribute.value });
       }
     }
+
+    // Look in parent elements if the db.pk or db.name is missing
+    if (this.isUnicorn && !isEmpty(this.field)) {
+      const dbAttrs = ["pk", "name"];
+      let elToCheck = this;
+
+      // Look for `db.pk` and `db.name`
+      dbAttrs.forEach((attr) => {
+        elToCheck = this;
+
+        while (isEmpty(this.db[attr])) {
+          if (elToCheck.el.getAttribute("unicorn:checksum")) {
+            // A litte hacky, but stop looking after you hit the beginning of the component
+            break;
+          }
+
+          if (elToCheck.isUnicorn && hasValue(elToCheck.db[attr])) {
+            this.db[attr] = elToCheck.db[attr];
+          }
+
+          elToCheck = elToCheck.parent;
+        }
+      });
+
+      // Look for model.name
+      elToCheck = this;
+
+      while (isEmpty(this.model.name)) {
+        if (elToCheck.el.getAttribute("unicorn:checksum")) {
+          // A litte hacky, but stop looking after you hit the beginning of the component
+          break;
+        }
+
+        if (elToCheck.isUnicorn && hasValue(elToCheck.model.name)) {
+          this.model.name = elToCheck.model.name;
+        }
+
+        elToCheck = elToCheck.parent;
+      }
+    }
   }
 
   /**
@@ -93,6 +155,22 @@ export class Element {
     this.el.focus();
   }
 
+  /**
+   * A key that takes into consideration the db name and pk.
+   */
+  dbKey() {
+    return generateDbKey(this);
+  }
+
+  /**
+   * Check if another `Element` is the same as this `Element`.
+   * @param {Element} other
+   */
+  isSame(other) {
+    // Use isSameNode (not isEqualNode) because we want to check the nodes reference the same object
+    return this.el.isSameNode(other.el);
+  }
+
   /**
    * Gets the value from the element.
    */
@@ -119,6 +197,10 @@ export class Element {
    * Sets the value of an element. Tries to deal with HTML weirdnesses.
    */
   setValue(val) {
+    if (isEmpty(this.el.type)) {
+      return;
+    }
+
     if (this.el.type.toLowerCase() === "radio") {
       // Handle radio buttons
       if (this.el.value === val) {
